////////////////////////////////////////////////////////////////////////////////
// PIC USB Discriptor (Custom HID)					Create Date	2015/11/08
//			for PIC16 / PIC18						Last Update	2018/04/04
// file name :  usbdesc.h									Saka Softwares
// language  :  XC8 (v1.45)
////////////////////////////////////////////////////////////////////////////////
// USBの機能に合わせて変更が必要です。
// 各ディスクリプタのサイズは256byte以内に納めてください。
////////////////////////////////////////////////////////////////////////////////
#ifndef _USB_DESCRIPTOR_H
#define	_USB_DESCRIPTOR_H
// Warning Disable /////////////////////////////////////////////////////////////
// 下記のワーニングを抑制しています。エラーレベル -3 でコンパイルできます。
#pragma warning disable 751		//warning: (751) arithmetic overflow in constant expression
////////////////////////////////////////////////////////////////////////////////
// スタンダード リクエスト
#define USB_MAX_DSC_CFG			1		// 設定するコンフィグ ディスクリプタ数
#define USB_MAX_DSC_INTF		1		// 設定するインターフェース ディスクリプタ数

// 各デスクプリタのサイズ
#define	DEVDSC_SIZE				0x12	// デバイス デスクプリタ
#define	CFGDSC_SIZE				0x09	// コンフィグ デスクプリタ
#define	IFDSC_SIZE				0x09	// インターフェース デスクプリタ
#define	HIDDSC_SIZE				0x09	// HID デスクプリタ
#define	EPDSC_SIZE				0x07	// エイドポイント デスクプリタ
#define	LANGDSC_SIZE			0x04	// 言語 ディスクリプタ

#define DSC_DEV					0x01	// デバイス ディスクプリタ タイプ
#define DSC_CFG					0x02	// コンフィグ ディスクプリタ タイプ
#define DSC_STR					0x03	// ストリング ディスクプリタ タイプ
#define DSC_INTF				0x04	// インターフェース ディスクプリタ タイプ
#define DSC_EP					0x05	// エンドポイント ディスクプリタ タイプ
#define DSC_DQ					0x06	// Device Qualitier
#define DSC_OSC					0x07	// Other Speed Configuratiln

#define DSC_EP_CTRL				0x00	// コントロール転送
#define DSC_EP_ISO				0x01	// アイソクロナス転送
#define DSC_EP_BULK				0x02	// バルク転送
#define DSC_EP_INT				0x03	// インターラプト転送

#define DSC_EP_IN				0x80	// エンドポイントIN
#define DSC_EP_OUT				0x00	// エンドポイントOUT

////////////////////////////////////////////////////////////////////////////////
// クラスリクエスト (HID)
#define HID_INTF_ID				0		// HID インターフェース ID
#define HID_NUM_OF_DSC			1		// クラスディスクリプタの数

// Class-Specific Requests
#define GET_REPORT				0x01
#define GET_IDLE				0x02
#define GET_PROTOCOL			0x03
#define SET_REPORT				0x09
#define SET_IDLE				0x0A
#define SET_PROTOCOL			0x0B

// Class Descriptor Types
#define DSC_HID					0x21
#define DSC_RPT					0x22
#define DSC_PHY					0x23

// Protocol Selection
#define BOOT_PROTOCOL			0x00
#define RPT_PROTOCOL			0x01

// HID Interface Class Code
#define HID_INTF				0x03

// HID Interface Class SubClass Codes
#define BOOT_INTF_SUBCLASS		0x01

// HID Interface Class Protocol Codes
#define HID_PROTOCOL_NONE		0x00
#define HID_PROTOCOL_KEYBOARD	0x01
#define HID_PROTOCOL_MOUSE 		0x02

// ディスクリプタ中のワード表記(warning: (751)を抑制)
#define DESC_WORD(a) (uint8_t)(a & 0xFF),(uint8_t)((a >> 8) & 0xFF)

////////////////////////////////////////////////////////////////////////////////
// 64byte 送受信用 リポートディスクリプタ
const uint8_t hid_rpt01[]={
	0x06, 0x00, 0xFF,		// Usage Page = 0xFF00 (ベンダー定義ページ)
	0x09, 0x01,				// Usage (用法 1)
	0xA1, 0x01,				// Collection (アプリケーション)
	0x19, 0x01,             //		用法 最小値
	0x29, 0x40,             //		用法 最大値 : 1～64 送信数
	0x15, 0x00,             //		論理的最小値
	0x26, 0xFF, 0x00,       //		論理的最大値 (0から255の転送データ値)
	0x75, 0x08,             //		レポートサイズ		8ビット幅(1byte)
	0x95, 0x40,             //		レポートカウント		64カウント (レポートサイズ * 64)
	0x81, 0x00,             //		Input (送信設定)
	0x19, 0x01,             //		用法 最小値
	0x29, 0x40,             //		用法 最大値 : 1～64 受信数
	0x91, 0x00,             //      Output (受信設定): レポートはInputと共用
	0xC0                    // End Collection
};    

////////////////////////////////////////////////////////////////////////////////
// デバイスディスクリプタ
const uint8_t device_dsc[] = {
	DEVDSC_SIZE,					// ディスクリプタの長さ(0x12)
	DSC_DEV,						// ディスクリプタのタイプ (デバイス)
	DESC_WORD( 0x0200 ),			// BCD表現のUSB仕様リリース番号(Ver2.00)
	0x00,							// クラスコード
	0x00,							// サブクラスコード
	0x00,							// プロトコルコード
	EP0_BUFF_SIZE,					// EP0の最大パケットサイズ
	DESC_WORD( MY_VID ),			// ベンダー ID
	DESC_WORD( MY_PID ),			// プロダクト ID
	DESC_WORD( MY_RLS ),			// BCD表現のデバイスリリース番号
	0x01,							// 製造者名のインデックス
	0x02,							// 製品名のインデックス
	0x00,							// 製品番号名のインデックス
	USB_MAX_DSC_CFG					// 構成可能な数 (Configの数)
};

////////////////////////////////////////////////////////////////////////////////
// 構成(コンフィグレーション)ディスクリプタ
const uint8_t cfg_dsc01[] = {
    // コンフィグレーション ディスクリプタ
	CFGDSC_SIZE,					// ディスクリプタの長さ(0x09)
	DSC_CFG,						// ディスクリプタのタイプ (構成)
	DESC_WORD(	CFGDSC_SIZE	+		// 構成全体(cofig, IF,etc)の長さ(可変)		
				IFDSC_SIZE	+
				HIDDSC_SIZE +		//(usbhiddrv.cは機能を制限している為、
				EPDSC_SIZE	+		// 構成全体を256Byte以内に設定してください。)
				EPDSC_SIZE	),
	USB_MAX_DSC_INTF,				// 構成が持つインターフェースの数
	1,								// この構成のインデックス番号(1以上)
	0,								// 構成名のインデックス
	CFG_ATTRIBUTES,					// 構成の特性 (ビット単位)
	CFG_BUSCURRENT,					// 最大バス消費電流量

    // インターフェース ディスクリプタ
	IFDSC_SIZE,						// ディスクリプタの長さ(0x09)
	DSC_INTF,						// ディスクリプタのタイプ (インターフェース)
	HID_INTF_ID,					// インターフェース番号
	0,								// インターフェース代替え用番号
	2,								// IFが使用するEPの数
	HID_INTF,						// IFのクラスコード
	0,								// IFサブクラスコード
	0,								// IFのプロトコルコード
	0,								// IFのインデックス

	// HID クラス ディスクリプタ
	HIDDSC_SIZE,					// ディスクリプタの長さ(0x09)
	DSC_HID,						// ディスクリプタのタイプ (HID)
	DESC_WORD(0x0111),				// BCD表現のHIDバージョン (Ver1.11)
	0x00,							// 地域固有デバイスのための国識別番号
	HID_NUM_OF_DSC,					// クラスディスクリプタの数
	DSC_RPT,						// クラスディスクリプタの型
	DESC_WORD(sizeof(hid_rpt01)),	// レポートディスクリプタのサイズ
    
    // EP1 in ディスクリプタ
	EPDSC_SIZE,						// ディスクリプタの長さ(0x07)
	DSC_EP,							// ディスクリプタのタイプ (エンドポイント)
	DSC_EP_IN + 1,					// エンドポイント アドレス
	DSC_EP_INT,						// エンドポイントの属性
	DESC_WORD(EP1_IN_BUFF_SIZE),	// ペイロードサイズ
    1,								// データ通信間隔 (ms)

    // EP1 out ディスクリプタ
	EPDSC_SIZE,						// ディスクリプタの長さ(0x07)
	DSC_EP,							// ディスクリプタのタイプ (エンドポイント)
	DSC_EP_OUT + 1,					// エンドポイント アドレス
	DSC_EP_INT,						// エンドポイントの属性
	DESC_WORD(EP1_OUT_BUFF_SIZE),	// ペイロードサイズ
    1,								// データ通信間隔 (ms)
};

////////////////////////////////////////////////////////////////////////////////
// 言語コード名 ディスクリプタ
const struct{uint8_t bLength;uint8_t bDscType;uint16_t string[1];}sd000={
	sizeof(sd000), DSC_STR, { 0x0409 }
};

////////////////////////////////////////////////////////////////////////////////
// 製造者名 ディスクリプタ
const struct{uint8_t bLength;uint8_t bDscType;uint16_t string[25];}sd001={
	sizeof(sd001), DSC_STR,
	{'M','i','c','r','o','c','h','i','p',' ',
	 'T','e','c','h','n','o','l','o','g','y',' ','I','n','c','.'}
};

////////////////////////////////////////////////////////////////////////////////
// 製品名 ディスクリプタ
const struct{uint8_t bLength;uint8_t bDscType;uint16_t string[22];}sd002={
	sizeof(sd002), DSC_STR,
	{'S','i','m','p','l','e',' ','H','I','D',' ',
	'D','e','v','i','c','e',' ','D','e','m','o'}
};	

#endif	//_USB_DESCRIPTOR_H
